<?php

/**
 * This is the model class for table "hrd_m_dosen".
 *
 * The followings are the available columns in table 'hrd_m_dosen':
 * @property integer $nidn
 * @property integer $nip
 * @property string $pend_akhir
 * @property string $prodi
 * @property string $gbk_1
 * @property string $gbk_2
 * @property string $created_by
 * @property string $created_date
 * @property string $created_ip
 * @property string $updated_by
 * @property string $updated_date
 * @property string $updated_ip
 *
 * The followings are the available model relations:
 * @property HrdHRiwayatPendidikan[] $hrdHRiwayatPendidikans
 * @property HrdHRiwayatPengajaran[] $hrdHRiwayatPengajarans
 * @property HrdMPegawai $nip0
 */
class HrdMDosen extends CActiveRecord
{
	/**
	 * Returns the static model of the specified AR class.
	 * @param string $className active record class name.
	 * @return HrdMDosen the static model class
	 */
	public static function model($className=__CLASS__)
	{
		return parent::model($className);
	}

	/**
	 * @return string the associated database table name
	 */
	public function tableName()
	{
		return 'hrd_m_dosen';
	}

	/**
	 * @return array validation rules for model attributes.
	 */
	public function rules()
	{
		// NOTE: you should only define rules for those attributes that
		// will receive user inputs.
		return array(
			array('nidn', 'required'),
			array('nidn, nip', 'numerical', 'integerOnly'=>true),
			array('pend_akhir, created_ip, updated_ip', 'length', 'max'=>200),
			array('prodi', 'length', 'max'=>45),
			array('gbk_1, gbk_2', 'length', 'max'=>50),
			array('created_by, updated_by', 'length', 'max'=>15),
			array('created_date, updated_date', 'safe'),
			// The following rule is used by search().
			// Please remove those attributes that should not be searched.
			array('nidn, nip, pend_akhir, prodi, gbk_1, gbk_2, created_by, created_date, created_ip, updated_by, updated_date, updated_ip', 'safe', 'on'=>'search'),
		);
	}

	/**
	 * @return array relational rules.
	 */
	public function relations()
	{
		// NOTE: you may need to adjust the relation name and the related
		// class name for the relations automatically generated below.
		return array(
			'hrdHRiwayatPendidikans' => array(self::HAS_MANY, 'HrdHRiwayatPendidikan', 'nidn'),
			'hrdHRiwayatPengajarans' => array(self::HAS_MANY, 'HrdHRiwayatPengajaran', 'nidn'),
			'nip0' => array(self::BELONGS_TO, 'HrdMPegawai', 'nip'),
		);
	}

	/**
	 * @return array customized attribute labels (name=>label)
	 */
	public function attributeLabels()
	{
		return array(
			'nidn' => 'Nidn',
			'nip' => 'Nip',
			'pend_akhir' => 'Pend Akhir',
			'prodi' => 'Prodi',
			'gbk_1' => 'Gbk 1',
			'gbk_2' => 'Gbk 2',
			'created_by' => 'Created By',
			'created_date' => 'Created Date',
			'created_ip' => 'Created Ip',
			'updated_by' => 'Updated By',
			'updated_date' => 'Updated Date',
			'updated_ip' => 'Updated Ip',
		);
	}

	/**
	 * Retrieves a list of models based on the current search/filter conditions.
	 * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
	 */
	public function search()
	{
		// Warning: Please modify the following code to remove attributes that
		// should not be searched.

		$criteria=new CDbCriteria;

		$criteria->compare('nidn',$this->nidn);
		$criteria->compare('nip',$this->nip);
		$criteria->compare('pend_akhir',$this->pend_akhir,true);
		$criteria->compare('prodi',$this->prodi,true);
		$criteria->compare('gbk_1',$this->gbk_1,true);
		$criteria->compare('gbk_2',$this->gbk_2,true);
		$criteria->compare('created_by',$this->created_by,true);
		$criteria->compare('created_date',$this->created_date,true);
		$criteria->compare('created_ip',$this->created_ip,true);
		$criteria->compare('updated_by',$this->updated_by,true);
		$criteria->compare('updated_date',$this->updated_date,true);
		$criteria->compare('updated_ip',$this->updated_ip,true);

		return new CActiveDataProvider($this, array(
			'criteria'=>$criteria,
		));
	}

    protected function afterValidate(){
        if ($this->isNewRecord){
            $this->created_date=new CDbExpression('NOW()');
            $this->created_by=Yii::app()->user->getId();
            $this->created_ip=Yii::app()->request->getUserHostAddress();


        }else{
            $this->updated_date=new CDbExpression('NOW()');
            $this->updated_by=Yii::app()->user->getId();
            $this->updated_ip=Yii::app()->request->getUserHostAddress();
        }
    }
}